@@ -22,6 +22,10 @@
 
 #include "evolution-webcal-notify.h"
 
+#define WEBCAL_BASE_URI "webcal://"
+#define CALENDAR_SOURCES_KEY "/apps/evolution/calendar/sources"
+#define TASKS_SOURCES_KEY "/apps/evolution/tasks/sources"
+
 typedef struct {
   GtkWidget * dialog;
   GtkWidget * image;
@@ -36,7 +40,7 @@ enum {
 };
 
 static void e_webcal_icon_theme_changed (GnomeIconTheme * theme,
-					    EIcalShareDialog * dialog) {
+					 EIcalShareDialog * dialog) {
   gchar * icofile;
 
   icofile = gnome_icon_theme_lookup_icon (dialog->theme, "stock_web-calendar",
@@ -50,7 +54,7 @@ static void e_webcal_icon_theme_changed (GnomeIconTheme * theme,
 }
 
 static void e_webcal_change_adjustment (GtkMenuShell * shell,
-					   GtkWidget * spinner) {
+					GtkWidget * spinner) {
   GtkWidget * widget;
 
   widget = gtk_menu_get_active (GTK_MENU (shell));
@@ -92,7 +96,7 @@ static gint e_webcal_option_menu_get (GtkOptionMenu * menu) {
 }
 
 void e_webcal_display_error (const gchar * title, const gchar * message,
-				GtkWidget * parent) {
+			     GtkWidget * parent) {
   GtkWidget * dialog;
   GtkWidget * label;
   GtkWidget * image;
@@ -165,31 +169,59 @@ void e_webcal_display_error (const gchar * title, const gchar * message,
 }
 
 void e_webcal_query_user (const gchar * name, const gchar * desc,
-			     const gchar * caluri) {
+                          const gchar * caluri,
+                          gboolean has_events, gboolean has_tasks) {
   EIcalShareDialog * dialog;
   GtkObject * spinadj;
   GtkWidget * tspinb, * toption;
   GtkWidget * menu, * mitem;
   GtkWidget * vbox, * hbox;
   GtkWidget * label, * button;
+  SoupUri * tmpuri;
+  gchar * tmpname;
   gchar * icofile, * mrkname, * ref_str;
   gint ref_timeout, ref_multi;
   ESource * source;
-  ESourceGroup * group;
-  ESourceList * sources;
+  ESourceGroup * events_group = NULL;
+  ESourceGroup * tasks_group = NULL;
+  ESourceList * events_sources, * tasks_sources;
   GSList * l;
 
-  sources = e_source_list_new_for_gconf_default ("/apps/evolution/calendar/sources");
+  events_sources = e_source_list_new_for_gconf_default (CALENDAR_SOURCES_KEY);
 
-  for (l = e_source_list_peek_groups (sources); l != NULL; l = l->next) {
-    group = l->data;
-
-    if (!strcmp (e_source_group_peek_base_uri (group), "webcal://")) {
+  for (l = e_source_list_peek_groups (events_sources); l != NULL; l = l->next) {
+    if (!strcmp (e_source_group_peek_base_uri (l->data), WEBCAL_BASE_URI)) {
+      events_group = l->data;
       break;
     }
   }
 
-  source = e_source_new (name, caluri);
+  if (!events_group) {
+    /* Create the Webcal source group */
+    events_group = e_source_group_new (_("On The Web"), WEBCAL_BASE_URI);
+    e_source_list_add_group (events_sources, events_group, -1);
+  }
+
+  tasks_sources  = e_source_list_new_for_gconf_default (TASKS_SOURCES_KEY);
+
+  for (l = e_source_list_peek_groups (tasks_sources); l != NULL; l = l->next) {
+    if (!strcmp (e_source_group_peek_base_uri (l->data), WEBCAL_BASE_URI)) {
+      tasks_group = l->data;
+      break;
+    }
+  }
+
+  if (!tasks_group) {
+    /* Create the Webcal source group */
+    tasks_group = e_source_group_new (_("On The Web"), WEBCAL_BASE_URI);
+    e_source_list_add_group (tasks_sources, tasks_group, -1);
+  }
+
+
+  tmpuri = soup_uri_new (caluri);
+  tmpname = g_path_get_basename (tmpuri->path);
+
+  source = e_source_new (name ? name : tmpname, caluri);
 
   dialog = g_new0 (EIcalShareDialog, 1);
 
@@ -276,7 +308,7 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   gtk_widget_show (vbox);
 
   /* Name */
-  mrkname = g_strdup_printf ("<b>%s</b>", name ? name : g_basename (caluri));
+  mrkname = g_strdup_printf ("<b>%s</b>", name ? name : tmpname);
   label = gtk_label_new (mrkname);
   gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
   g_free (mrkname);
@@ -359,6 +391,8 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
 
 
   g_free (icofile);
+  g_free (tmpname);
+  soup_uri_free (tmpuri);
 
   switch (gtk_dialog_run (GTK_DIALOG (dialog->dialog))) {
   case GTK_RESPONSE_HELP:
@@ -383,20 +417,28 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
     e_source_set_property (source, "refresh", ref_str);
     g_free (ref_str);
 
-    if (!strncmp (caluri, "webcal://", 9)) {
+    if (!strncmp (caluri, WEBCAL_BASE_URI, 9)) {
       e_source_set_relative_uri (source, caluri + 9);
     } else if (!strncmp (caluri, "http://", 7)) {
       e_source_set_relative_uri (source, caluri + 7);
     }
 
-    e_source_group_add_source (group, source, -1);
-    e_source_list_sync (sources, NULL);
+    if (has_events) {
+      e_source_group_add_source (events_group, source, -1);
+      e_source_list_sync (events_sources, NULL);
+    }
+
+    if (has_tasks) {
+      e_source_group_add_source (tasks_group, source, -1);
+      e_source_list_sync (tasks_sources, NULL);
+    }
 
     /* We fall through to the CANCEL/default cases to handle all the unreffing
        and widget destruction */
   case GTK_RESPONSE_CANCEL:
   default:
-    g_object_unref (sources);
+    g_object_unref (events_sources);
+    g_object_unref (tasks_sources);
     g_object_unref (source);
 
     g_object_unref (dialog->theme);
